/************************************************************************
     File:        runbutton.h
		trivially updated in October 2007

     Author:     
                  Michael L Gleicher, gleicher@cs.wisc.edu
     Modifier
                  Yu-Chi Lai, yu-chi@cs.wisc.edu
     
     Comment:     written 10/16/99, Michael L Gleicher
		              This provides a utility routine to help with animation
						This file implements a "play" button. The idea is that you creat a 
						window that	does your drawing. When redrawing, this window looks at a 
						slider to see what time it is, so it knows what part of the animation to 
						draw.

						The slider must call the window's redraw function whenever the time changes.

						The play button, when pressed, advances the slider forward a timestep continually.
						In order to do this, it installs itself in the FlTk "Idle" loop so it gets called
						periodically.
  
  To use this in your program:
  When you set up your "main window" you probably have an OpenGL Window inside of
  it. 
  * After creating that window, create a RunSlider that refers to it
  * After creating the run slider that refers to it, create a runbutton that refers to that
  * Inside your windows draw function (or wherever), look at the value of the slider to see
    "what time it is"

  Note: 
  * The slider is advanced (roughly) 30 times per second
  * The slider goes from 0 to LENGTH in steps of 1. You can adjust the length
    of the slider as you need to

     CVS Info: Date checked in: $Date: 2007/10/22 15:38:30 $
               Revision number: $Revision: 1.3 $

*************************************************************************/

#ifndef RUNBUTTON_H
#define RUNBUTTON_H

// the FlTk headers have lots of warnings - these are bad, but there's not
// much we can do about them
#pragma warning(push)
#pragma warning(disable:4311)		// convert void* to long
#pragma warning(disable:4312)		// convert long to void*
#include <Fl/Fl_Value_Slider.H>
#include <Fl/Fl_Light_Button.H>
#pragma warning(pop)

// the time slider
// the only thing it does is create itself. all of its work happens
// in a callback it installs
class RunSlider  : public Fl_Value_Slider {
public:
	RunSlider(Fl_Widget* draw,		// what needs to be told to redraw
				  int length,			// how many frames is the animation
				  int x, int y, int w,int h);			// standard slider parameters

  };

// the run button - advances the slider periodically
// note: this is hard coded to 30 frames per second
class RunButton : public Fl_Light_Button {
public:
	RunButton(Fl_Slider* time,		// what to advance
			  int x, int y, int w, int h);
	~RunButton();					// remove the idle function

	Fl_Slider* slider;				// need to remember this
	long lastRedraw;				// time of last redraw
};

#endif
